@@ -392,7 +392,7 @@ DoGetDirection(int dx, int dy){
     float r;
     int i1, i2;
     /* on insignificant mickeys, flag 135 degrees */
-    if(abs(dx) < 2 && abs(dy < 2)){
+    if(abs(dx) < 2 && abs(dy) < 2){
 	/* first check diagonal cases */
 	if(dx > 0 && dy > 0)
 	    return 4+8+16;
