@@ -145,7 +145,7 @@ Bool dmxPictureInit(ScreenPtr pScreen, PictFormatPtr formats, int nformats)
     if (!miPictureInit(pScreen, formats, nformats))
 	return FALSE;
 
-    if (!dixRequestPrivate(dmxPictPrivateKey, sizeof(dmxPictPrivRec)))
+    if (!dixRegisterPrivateKey(&dmxPictPrivateKeyRec, PRIVATE_PICTURE, sizeof(dmxPictPrivRec)))
 	return FALSE;
 
     ps = GetPictureScreen(pScreen);
@@ -271,10 +271,12 @@ static int dmxProcRenderCreateGlyphSet(ClientPtr client)
 	/* Look up glyphSet that was just created ???? */
 	/* Store glyphsets from backends in glyphSet->devPrivate ????? */
 	/* Make sure we handle all errors here!! */
-	
-	glyphSet = SecurityLookupIDByType(client, stuff->gsid, GlyphSetType,
-					  DixDestroyAccess);
-	glyphPriv = xalloc(sizeof(dmxGlyphPrivRec));
+
+	dixLookupResourceByType((pointer*) &glyphSet,
+				stuff->gsid, GlyphSetType,
+				client, DixDestroyAccess);
+
+	glyphPriv = malloc(sizeof(dmxGlyphPrivRec));
 	if (!glyphPriv) return BadAlloc;
         glyphPriv->glyphSets = NULL;
         MAXSCREENSALLOC_RETURN(glyphPriv->glyphSets, BadAlloc);
@@ -314,8 +316,9 @@ static int dmxProcRenderFreeGlyphSet(ClientPtr client)
     REQUEST(xRenderFreeGlyphSetReq);
 
     REQUEST_SIZE_MATCH(xRenderFreeGlyphSetReq);
-    glyphSet = SecurityLookupIDByType(client, stuff->glyphset, GlyphSetType,
-				      DixDestroyAccess);
+    dixLookupResourceByType((pointer*) &glyphSet,
+			    stuff->glyphset, GlyphSetType,
+			    client, DixDestroyAccess);
 
     if (glyphSet && glyphSet->refcnt == 1) {
 	dmxGlyphPrivPtr  glyphPriv = DMX_GET_GLYPH_PRIV(glyphSet);
@@ -331,7 +334,7 @@ static int dmxProcRenderFreeGlyphSet(ClientPtr client)
 	}
 
         MAXSCREENSFREE(glyphPriv->glyphSets);
-	xfree(glyphPriv);
+	free(glyphPriv);
 	DMX_SET_GLYPH_PRIV(glyphSet, NULL);
     }
 
@@ -357,8 +360,9 @@ static int dmxProcRenderAddGlyphs(ClientPtr client)
 	CARD8           *bits;
 	int              nbytes;
 
-	glyphSet = SecurityLookupIDByType(client, stuff->glyphset,
-					  GlyphSetType, DixReadAccess);
+	dixLookupResourceByType((pointer*) &glyphSet,
+				stuff->glyphset, GlyphSetType,
+				client, DixReadAccess);
 	glyphPriv = DMX_GET_GLYPH_PRIV(glyphSet);
 
 	nglyphs = stuff->nglyphs;
@@ -369,7 +373,7 @@ static int dmxProcRenderAddGlyphs(ClientPtr client)
 		  sizeof(xRenderAddGlyphsReq) -
 		  (sizeof(CARD32) + sizeof(xGlyphInfo)) * nglyphs);
 
-        gidsCopy = xalloc(sizeof(*gidsCopy) * nglyphs);
+        gidsCopy = malloc(sizeof(*gidsCopy) * nglyphs);
         for (i = 0; i < nglyphs; i++) gidsCopy[i] = gids[i];
 
 	/* FIXME: Will this ever fail? */
@@ -387,7 +391,7 @@ static int dmxProcRenderAddGlyphs(ClientPtr client)
 		dmxSync(dmxScreen, FALSE);
 	    }
 	}
-        xfree(gidsCopy);
+        free(gidsCopy);
     }
 
     return ret;
@@ -400,8 +404,9 @@ static int dmxProcRenderFreeGlyphs(ClientPtr client)
     REQUEST(xRenderFreeGlyphsReq);
 
     REQUEST_AT_LEAST_SIZE(xRenderFreeGlyphsReq);
-    glyphSet = SecurityLookupIDByType(client, stuff->glyphset, GlyphSetType,
-				      DixWriteAccess);
+    dixLookupResourceByType((pointer*) &glyphSet,
+			    stuff->glyphset, GlyphSetType,
+			    client, DixWriteAccess);
 
     if (glyphSet) {
 	dmxGlyphPrivPtr  glyphPriv = DMX_GET_GLYPH_PRIV(glyphSet);
@@ -411,7 +416,7 @@ static int dmxProcRenderFreeGlyphs(ClientPtr client)
 
 	nglyphs = ((client->req_len << 2) - sizeof(xRenderFreeGlyphsReq)) >> 2;
 	if (nglyphs) {
-            gids    = xalloc(sizeof(*gids) * nglyphs);
+            gids    = malloc(sizeof(*gids) * nglyphs);
             for (i = 0; i < nglyphs; i++)
                 gids[i] = ((CARD32 *)(stuff + 1))[i];
             
@@ -424,7 +429,7 @@ static int dmxProcRenderFreeGlyphs(ClientPtr client)
 		    dmxSync(dmxScreen, FALSE);
 		}
 	    }
-            xfree(gids);
+            free(gids);
 	}
     }
 
@@ -472,14 +477,18 @@ static int dmxProcRenderCompositeGlyphs(ClientPtr client)
 	GlyphSetPtr        glyphSet;
 	dmxGlyphPrivPtr    glyphPriv;
 
-	pSrc = SecurityLookupIDByType(client, stuff->src, PictureType,
-				      DixReadAccess);
+	dixLookupResourceByType((pointer*) &pSrc,
+				stuff->src, PictureType,
+				client, DixReadAccess);
+
 	pSrcPriv = DMX_GET_PICT_PRIV(pSrc);
 	if (!pSrcPriv->pict)
 	    return ret;
 
-	pDst = SecurityLookupIDByType(client, stuff->dst, PictureType,
-				      DixWriteAccess);
+	dixLookupResourceByType((pointer*) &pDst,
+				stuff->dst, PictureType,
+				client, DixWriteAccess);
+
 	pDstPriv = DMX_GET_PICT_PRIV(pDst);
 	if (!pDstPriv->pict)
 	    return ret;
@@ -495,8 +504,9 @@ static int dmxProcRenderCompositeGlyphs(ClientPtr client)
 	    return ret;
 
 	if (stuff->maskFormat)
-	    pFmt = SecurityLookupIDByType(client, stuff->maskFormat,
-					  PictFormatType, DixReadAccess);
+	    dixLookupResourceByType((pointer*) &pFmt,
+				    stuff->maskFormat, PictFormatType,
+				    client, DixReadAccess);
 	else
 	    pFmt = NULL;
 
@@ -531,13 +541,13 @@ static int dmxProcRenderCompositeGlyphs(ClientPtr client)
 	/* The following only works for Render version > 0.2 */
 
 	/* All of the XGlyphElt* structure sizes are identical */
-	elts = xalloc(nelt * sizeof(XGlyphElt8));
+	elts = malloc(nelt * sizeof(XGlyphElt8));
 	if (!elts)
 	    return BadAlloc;
 
-	glyphs = xalloc(nglyph * size);
+	glyphs = malloc(nglyph * size);
 	if (!glyphs) {
-	    xfree(elts);
+	    free(elts);
 	    return BadAlloc;
 	}
 
@@ -546,8 +556,9 @@ static int dmxProcRenderCompositeGlyphs(ClientPtr client)
 	curGlyph = glyphs;
 	curElt = elts;
 
-	glyphSet = SecurityLookupIDByType(client, stuff->glyphset,
-					  GlyphSetType, DixReadAccess);
+	dixLookupResourceByType((pointer*) &glyphSet,
+				stuff->glyphset, GlyphSetType,
+				client, DixReadAccess);
 	glyphPriv = DMX_GET_GLYPH_PRIV(glyphSet);
 
 	while (buffer + sizeof(xGlyphElt) < end) {
@@ -555,10 +566,11 @@ static int dmxProcRenderCompositeGlyphs(ClientPtr client)
 	    buffer += sizeof(xGlyphElt);
 
 	    if (elt->len == 0xff) {
-		glyphSet = SecurityLookupIDByType(client,
-						  *((CARD32 *)buffer),
-						  GlyphSetType,
-						  DixReadAccess);
+		dixLookupResourceByType((pointer*) &glyphSet,
+					*((CARD32 *)buffer),
+					GlyphSetType,
+					client,
+					DixReadAccess);
 		glyphPriv = DMX_GET_GLYPH_PRIV(glyphSet);
 		buffer += 4;
 	    } else {
@@ -605,8 +617,8 @@ static int dmxProcRenderCompositeGlyphs(ClientPtr client)
 
 	dmxSync(dmxScreen, FALSE);
 
-	xfree(elts);
-	xfree(glyphs);
+	free(elts);
+	free(glyphs);
     }
 
     return ret;
@@ -622,8 +634,7 @@ static int dmxProcRenderSetPictureTransform(ClientPtr client)
     REQUEST(xRenderSetPictureTransformReq);
 
     REQUEST_SIZE_MATCH(xRenderSetPictureTransformReq);
-    VERIFY_PICTURE(pPicture, stuff->picture, client, DixWriteAccess,
-		   RenderErrBase + BadPicture);
+    VERIFY_PICTURE(pPicture, stuff->picture, client, DixWriteAccess);
 
     /* For the following to work with PanoramiX, it assumes that Render
      * wraps the ProcRenderVector after dmxRenderInit has been called.
@@ -663,8 +674,7 @@ static int dmxProcRenderSetPictureFilter(ClientPtr client)
     REQUEST(xRenderSetPictureFilterReq);
 
     REQUEST_AT_LEAST_SIZE(xRenderSetPictureFilterReq);
-    VERIFY_PICTURE(pPicture, stuff->picture, client, DixWriteAccess,
-		   RenderErrBase + BadPicture);
+    VERIFY_PICTURE(pPicture, stuff->picture, client, DixWriteAccess);
 
     /* For the following to work with PanoramiX, it assumes that Render
      * wraps the ProcRenderVector after dmxRenderInit has been called.
@@ -873,14 +883,14 @@ int dmxChangePictureClip(PicturePtr pPicture, int clipType,
 				       pPictPriv->pict, 0, 0, None);
 	} else if (pPicture->clientClip) {
 	    RegionPtr   pClip = pPicture->clientClip;
-	    BoxPtr      pBox  = REGION_RECTS(pClip);
-	    int         nBox  = REGION_NUM_RECTS(pClip);
+	    BoxPtr      pBox  = RegionRects(pClip);
+	    int         nBox  = RegionNumRects(pClip);
 	    XRectangle *pRects;
 	    XRectangle *pRect;
 	    int         nRects;
 
 	    nRects = nBox;
-	    pRects = pRect = xalloc(nRects * sizeof(*pRect));
+	    pRects = pRect = malloc(nRects * sizeof(*pRect));
 
 	    while (nBox--) {
 		pRect->x      = pBox->x1;
@@ -896,7 +906,7 @@ int dmxChangePictureClip(PicturePtr pPicture, int clipType,
 					    0, 0,
 					    pRects,
 					    nRects);
-	    xfree(pRects);
+	    free(pRects);
 	} else {
 	    XRenderSetPictureClipRectangles(dmxScreen->beDisplay,
 					    pPictPriv->pict,
@@ -1013,8 +1023,6 @@ void dmxValidatePicture(PicturePtr pPicture, Mask mask)
 	    attribs.poly_edge = pPicture->polyEdge;
 	if (mask & CPPolyMode)
 	    attribs.poly_mode = pPicture->polyMode;
-	if (mask & CPDither)
-	    attribs.dither = pPicture->dither;
 	if (mask & CPComponentAlpha)
 	    attribs.component_alpha = pPicture->componentAlpha;
 
