@@ -29,18 +29,16 @@ KdCardInfo  *kdCardInfo;
 
 KdCardInfo *
 KdCardInfoAdd (KdCardFuncs  *funcs,
-	       KdCardAttr   *attr,
 	       void	    *closure)
 {
     KdCardInfo	*ci, **prev;
 
-    ci = xcalloc (1, sizeof (KdCardInfo));
+    ci = calloc(1, sizeof (KdCardInfo));
     if (!ci)
 	return 0;
     for (prev = &kdCardInfo; *prev; prev = &(*prev)->next);
     *prev = ci;
     ci->cfuncs = funcs;
-    ci->attr = *attr;
     ci->closure = closure;
     ci->screenList = 0;
     ci->selected = 0;
@@ -68,7 +66,7 @@ KdCardInfoDispose (KdCardInfo *ci)
 	if (*prev == ci)
 	{
 	    *prev = ci->next;
-	    xfree (ci);
+	    free(ci);
 	    break;
 	}
 }
@@ -79,7 +77,7 @@ KdScreenInfoAdd (KdCardInfo *ci)
     KdScreenInfo    *si, **prev;
     int		    n;
 
-    si = xcalloc (1, sizeof (KdScreenInfo));
+    si = calloc(1, sizeof (KdScreenInfo));
     if (!si)
 	return 0;
     for (prev = &ci->screenList, n = 0; *prev; prev = &(*prev)->next, n++);
@@ -100,7 +98,7 @@ KdScreenInfoDispose (KdScreenInfo *si)
 	if (*prev == si)
 	{
 	    *prev = si->next;
-	    xfree (si);
+	    free(si);
 	    if (!ci->screenList)
 		KdCardInfoDispose (ci);
 	    break;
@@ -114,7 +112,7 @@ KdNewPointer (void)
     KdPointerInfo *pi;
     int i;
 
-    pi = (KdPointerInfo *)xcalloc(1, sizeof(KdPointerInfo));
+    pi = (KdPointerInfo *)calloc(1, sizeof(KdPointerInfo));
     if (!pi)
         return NULL;
 
@@ -138,40 +136,28 @@ KdFreePointer(KdPointerInfo *pi)
 {
     InputOption *option, *prev = NULL;
 
-    if (pi->name)
-        xfree(pi->name);
-    if (pi->path)
-        xfree(pi->path);
+    free(pi->name);
+    free(pi->path);
 
     for (option = pi->options; option; option = option->next) {
-        if (prev)
-            xfree(prev);
-        if (option->key)
-            xfree(option->key);
-        if (option->value)
-            xfree(option->value);
+        free(prev);
+        free(option->key);
+        free(option->value);
         prev = option;
     }
 
-    if (prev)
-        xfree(prev);
-    
-    xfree(pi);
+    free(prev);
+    free(pi);
 }
- 
+
 void
 KdFreeKeyboard(KdKeyboardInfo *ki)
 {
-    if (ki->name)
-        xfree(ki->name);
-    if (ki->path)
-        xfree(ki->path);
-    if (ki->xkbRules)
-        xfree(ki->xkbRules);
-    if (ki->xkbModel)
-        xfree(ki->xkbModel);
-    if (ki->xkbLayout)
-        xfree(ki->xkbLayout);
+    free(ki->name);
+    free(ki->path);
+    free(ki->xkbRules);
+    free(ki->xkbModel);
+    free(ki->xkbLayout);
     ki->next = NULL;
-    xfree(ki);
+    free(ki);
 }
