@@ -1,3 +1,7 @@
+#ifdef HAVE_XORG_CONFIG_H
+#include "xorg-config.h"
+#endif
+
 #include "os.h"
 #include "xf86.h"
 #include "xf86Priv.h"
@@ -12,7 +16,6 @@
 #include <errno.h>
  
 #define ACPI_SOCKET  "/var/run/acpid.socket"
-#define ACPI_EVENTS  "/proc/acpi/event"
 
 #define ACPI_VIDEO_NOTIFY_SWITCH	0x80
 #define ACPI_VIDEO_NOTIFY_PROBE		0x81
@@ -20,11 +23,11 @@
 #define ACPI_VIDEO_NOTIFY_NEXT_OUTPUT	0x83
 #define ACPI_VIDEO_NOTIFY_PREV_OUTPUT	0x84
 
-#define ACPI_VIDEO_NOTIFY_CYCLE_BRIGHTNESS	0x82
-#define	ACPI_VIDEO_NOTIFY_INC_BRIGHTNESS	0x83
-#define ACPI_VIDEO_NOTIFY_DEC_BRIGHTNESS	0x84
-#define ACPI_VIDEO_NOTIFY_ZERO_BRIGHTNESS	0x85
-#define ACPI_VIDEO_NOTIFY_DISPLAY_OFF		0x86
+#define ACPI_VIDEO_NOTIFY_CYCLE_BRIGHTNESS	0x85
+#define	ACPI_VIDEO_NOTIFY_INC_BRIGHTNESS	0x86
+#define ACPI_VIDEO_NOTIFY_DEC_BRIGHTNESS	0x87
+#define ACPI_VIDEO_NOTIFY_ZERO_BRIGHTNESS	0x88
+#define ACPI_VIDEO_NOTIFY_DISPLAY_OFF		0x89
 
 #define ACPI_VIDEO_HEAD_INVALID		(~0u - 1)
 #define ACPI_VIDEO_HEAD_END		(~0u)
@@ -33,6 +36,20 @@ static void lnxCloseACPI(void);
 static pointer ACPIihPtr = NULL;
 PMClose lnxACPIOpen(void);
 
+/* in milliseconds */
+#define ACPI_REOPEN_DELAY 1000
+
+static CARD32
+lnxACPIReopen(OsTimerPtr timer, CARD32 time, pointer arg)
+{
+    if (lnxACPIOpen()) {
+	TimerFree(timer);
+	return 0;
+    }
+
+    return ACPI_REOPEN_DELAY;
+}
+
 #define LINE_LENGTH 80
 
 static int
@@ -43,17 +60,27 @@ lnxACPIGetEventFromOs(int fd, pmEvent *events, int num)
 
     memset(ev, 0, LINE_LENGTH);
 
-    n = read( fd, ev, LINE_LENGTH );
+    do {
+	n = read( fd, ev, LINE_LENGTH );
+    } while ((n == -1) && (errno == EAGAIN || errno == EINTR));
 
+    if (n <= 0) {
+	lnxCloseACPI();
+	TimerSet(NULL, 0, ACPI_REOPEN_DELAY, lnxACPIReopen, NULL);
+	return 0;
+    }
+    /* FIXME: this only processes the first read ACPI event & might break
+     * with interrupted reads. */
+    
     /* Check that we have a video event */
-    if (strstr(ev, "video") == ev) {
+    if (!strncmp(ev, "video", 5)) {
 	char *video = NULL;
 	char *GFX = NULL;
 	char *notify = NULL;
 	char *data = NULL; /* doesn't appear to be used in the kernel */
 	unsigned long int notify_l, data_l;
 
-	video = strtok(ev, "video");
+	video = strtok(ev, " ");
 
 	GFX = strtok(NULL, " ");
 #if 0
@@ -72,26 +99,19 @@ lnxACPIGetEventFromOs(int fd, pmEvent *events, int num)
 	ErrorF("data: 0x%lx\n",data_l);
 #endif
 
-	/* We currently don't differentiate between any event */
+	/* Differentiate between events */
 	switch (notify_l) {
 		case ACPI_VIDEO_NOTIFY_SWITCH:
-			break;
-		case ACPI_VIDEO_NOTIFY_PROBE:
-			break;
 		case ACPI_VIDEO_NOTIFY_CYCLE:
-			break;
 		case ACPI_VIDEO_NOTIFY_NEXT_OUTPUT:
-			break;
 		case ACPI_VIDEO_NOTIFY_PREV_OUTPUT:
-			break;
+		    events[0] = XF86_APM_CAPABILITY_CHANGED;
+		    return 1;
+		case ACPI_VIDEO_NOTIFY_PROBE:
+		    return 0;
 		default:
-			break;
+		    return 0;
 	}
-
-	/* Deal with all ACPI events as a capability change */
-        events[0] = XF86_APM_CAPABILITY_CHANGED;
-
-	return 1;
     }
     
     return 0;
@@ -113,40 +133,33 @@ lnxACPIOpen(void)
     int fd;    
     struct sockaddr_un addr;
     int r = -1;
+    static int warned = 0;
 
-#ifdef DEBUG
-    ErrorF("ACPI: OSPMOpen called\n");
-#endif
+    DebugF("ACPI: OSPMOpen called\n");
     if (ACPIihPtr || !xf86Info.pmFlag)
 	return NULL;
    
-#ifdef DEBUG
-    ErrorF("ACPI: Opening device\n");
-#endif
+    DebugF("ACPI: Opening device\n");
     if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) > -1) {
 	memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_UNIX;
 	strcpy(addr.sun_path, ACPI_SOCKET);
 	if ((r = connect(fd, (struct sockaddr*)&addr, sizeof(addr))) == -1) {
+	    if (!warned)
+		xf86MsgVerb(X_WARNING,3,"Open ACPI failed (%s) (%s)\n",
+		            ACPI_SOCKET, strerror(errno));
+	    warned = 1;
 	    shutdown(fd, 2);
 	    close(fd);
-	    fd = -1;
-	}
-    }
-
-    /* acpid's socket isn't available, so try going direct */
-    if (fd == -1) {
-        if ((fd = open(ACPI_EVENTS, O_RDONLY)) < 0) {
-	    xf86MsgVerb(X_WARNING,3,"Open ACPI failed (%s) (%s)\n", ACPI_EVENTS,
-	    	strerror(errno));
 	    return NULL;
     	}
     }
 
     xf86PMGetEventFromOs = lnxACPIGetEventFromOs;
     xf86PMConfirmEventToOs = lnxACPIConfirmEventToOs;
-    ACPIihPtr = xf86AddInputHandler(fd,xf86HandlePMEvents,NULL);
-    xf86MsgVerb(X_INFO,3,"Open ACPI successful (%s)\n", (r != -1) ? ACPI_SOCKET : ACPI_EVENTS);
+    ACPIihPtr = xf86AddGeneralHandler(fd,xf86HandlePMEvents,NULL);
+    xf86MsgVerb(X_INFO,3,"Open ACPI successful (%s)\n", ACPI_SOCKET);
+    warned = 0;
 
     return lnxCloseACPI;
 }
@@ -155,15 +168,12 @@ static void
 lnxCloseACPI(void)
 {
     int fd;
-    
-#ifdef DEBUG
-   ErrorF("ACPI: Closing device\n");
-#endif
+
+    DebugF("ACPI: Closing device\n");
     if (ACPIihPtr) {
-	fd = xf86RemoveInputHandler(ACPIihPtr);
+	fd = xf86RemoveGeneralHandler(ACPIihPtr);
 	shutdown(fd, 2);
 	close(fd);
 	ACPIihPtr = NULL;
     }
 }
-
