@@ -35,13 +35,13 @@
 #include "xf86Priv.h"
 #include "xf86_OSlib.h"
 
-_X_EXPORT Bool
+Bool
 xf86EnableIO()
 {
 	return TRUE;
 }
 
-_X_EXPORT void
+void
 xf86DisableIO()
 {
 	return;
