@@ -95,11 +95,8 @@ static miZeroArcPtRec oob = {65536, 65536, 0};
  *
  */
 
-_X_EXPORT Bool
-miZeroArcSetup(arc, info, ok360)
-    xArc *arc;
-    miZeroArcRec *info;
-    Bool ok360;
+Bool
+miZeroArcSetup(xArc *arc, miZeroArcRec *info, Bool ok360)
 {
     int l;
     int angle1, angle2;
@@ -705,18 +702,14 @@ miZeroArcDashPts(
     dinfo->dashOffset = pGC->dash[dinfo->dashIndex] - dashRemaining;
 }
 
-_X_EXPORT void
-miZeroPolyArc(pDraw, pGC, narcs, parcs)
-    DrawablePtr	pDraw;
-    GCPtr	pGC;
-    int		narcs;
-    xArc	*parcs;
+void
+miZeroPolyArc(DrawablePtr pDraw, GCPtr pGC, int narcs, xArc *parcs)
 {
     int maxPts = 0;
     int n, maxw = 0;
     xArc *arc;
     int i;
-    DDXPointPtr points, pts, oddPts;
+    DDXPointPtr points, pts, oddPts = NULL;
     DDXPointPtr pt;
     int numPts;
     Bool dospans;
@@ -744,7 +737,7 @@ miZeroPolyArc(pDraw, pGC, narcs, parcs)
     dospans = (pGC->fillStyle != FillSolid);
     if (dospans)
     {
-	widths = (int *)xalloc(sizeof(int) * numPts);
+	widths = malloc(sizeof(int) * numPts);
 	if (!widths)
 	    return;
 	maxw = 0;
@@ -761,12 +754,12 @@ miZeroPolyArc(pDraw, pGC, narcs, parcs)
 		   (unsigned char *) pGC->dash, (int)pGC->numInDashList,
 		   &dinfo.dashOffsetInit);
     }
-    points = (DDXPointPtr)xalloc(sizeof(DDXPointRec) * numPts);
+    points = malloc(sizeof(DDXPointRec) * numPts);
     if (!points)
     {
 	if (dospans)
 	{
-	    xfree(widths);
+	    free(widths);
 	}
 	return;
     }
@@ -810,7 +803,9 @@ miZeroPolyArc(pDraw, pGC, narcs, parcs)
 	    if ((pGC->fillStyle == FillSolid) ||
 		(pGC->fillStyle == FillStippled))
 	    {
-		DoChangeGC(pGC, GCForeground, (XID *)&pGC->bgPixel, 0);
+		ChangeGCVal gcval;
+		gcval.val = pGC->bgPixel;
+		ChangeGC(NullClient, pGC, GCForeground, &gcval);
 		ValidateGC(pDraw, pGC);
 	    }
 	    pts = &points[numPts >> 1];
@@ -838,14 +833,16 @@ miZeroPolyArc(pDraw, pGC, narcs, parcs)
 	    if ((pGC->fillStyle == FillSolid) ||
 		(pGC->fillStyle == FillStippled))
 	    {
-		DoChangeGC(pGC, GCForeground, &fgPixel, 0);
+		ChangeGCVal gcval;
+		gcval.val = fgPixel;
+		ChangeGC(NullClient, pGC, GCForeground, &gcval);
 		ValidateGC(pDraw, pGC);
 	    }
 	}
     }
-    xfree(points);
+    free(points);
     if (dospans)
     {
-	xfree(widths);
+	free(widths);
     }
 }
